##Procedure Header
# Name:
#    ixiangpf::emulation_isis_srv6_config
#
# Description:
#    This procedure is used to create and modify ISIS SRv6 realted fields in ISIS Router.
#
# Synopsis:
#    ixiangpf::emulation_isis_srv6_config
#        -mode                                                CHOICES create modify disable
#        -handle                                              ANY
#x       [-locator_count                                      RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-interface_adj_si_count                             RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-locator_entry_active                               CHOICES 0 1]
#x       [-srv6_metric                                        RANGE 0-4294967295
#x                                                            DEFAULT 10]
#x       [-srv6_algorithm                                     RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-d_bit                                              CHOICES 0 1]
#x       [-reserved_flags                                     HEX]
#x       [-locator_size                                       RANGE 0-128
#x                                                            DEFAULT 64]
#x       [-srv6_locator                                       IPV6]
#x       [-advertise_locator_as_prefix                        CHOICES 0 1]
#x       [-prefix_length                                      RANGE 0-128
#x                                                            DEFAULT 64]
#x       [-route_metric                                       RANGE 0-16777215
#x                                                            DEFAULT 10]
#x       [-srv6_redistribution                                CHOICES up down]
#x       [-route_origin                                       CHOICES internal external]
#x       [-sid_count                                          NUMERIC]
#x       [-srv6_end_sid_active                                CHOICES 0 1]
#x       [-flags                                              HEX]
#x       [-end_point_function                                 RANGE 1-255
#x                                                            DEFAULT 4]
#x       [-pseudo_srv6_end_point_function                     CHOICES end_nopsp_nousp
#x                                                            CHOICES end_psp
#x                                                            CHOICES end_usp
#x                                                            CHOICES end_psp_usp
#x                                                            CHOICES endt_nopsp_nousp
#x                                                            CHOICES endt_psp
#x                                                            CHOICES endt_usp
#x                                                            CHOICES endt_psp_usp
#x                                                            CHOICES endx_nopsp_nousp
#x                                                            CHOICES endx_psp
#x                                                            CHOICES endx_usp
#x                                                            CHOICES endx_psp_usp
#x                                                            CHOICES enddx6
#x                                                            CHOICES enddt6
#x                                                            CHOICES endotp
#x                                                            CHOICES custom]
#x       [-sid                                                IPV6]
#x       [-include_srv6_sid_structure_sub_sub_tlv             CHOICES 0 1]
#x       [-locator_block_length                               RANGE 0-128
#x                                                            DEFAULT 40]
#x       [-locator_node_length                                RANGE 0-128
#x                                                            DEFAULT 24]
#x       [-function_length                                    RANGE 0-128
#x                                                            DEFAULT 16]
#x       [-argument_length                                    RANGE 0-128
#x                                                            DEFAULT 0]
#x       [-advertise_custom_sub_tlv                           CHOICES 0 1]
#x       [-custom_sub_tlv                                     HEX]
#x       [-srv6_adj_sid_active                                CHOICES 0 1]
#x       [-ipv6_adj_sid                                       IPV6]
#x       [-b_flag                                             CHOICES 0 1]
#x       [-s_flag                                             CHOICES 0 1]
#x       [-p_flag                                             CHOICES 0 1]
#x       [-reserved                                           HEX]
#x       [-interface_algorithm                                RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-weight                                             RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-srv6_adj_end_point_function                        RANGE 1-255
#x                                                            DEFAULT 4]
#x       [-pseudo_srv6_adj_end_point_function                 CHOICES end_nopsp_nousp
#x                                                            CHOICES end_psp
#x                                                            CHOICES end_usp
#x                                                            CHOICES end_psp_usp
#x                                                            CHOICES endt_nopsp_nousp
#x                                                            CHOICES endt_psp
#x                                                            CHOICES endt_usp
#x                                                            CHOICES endt_psp_usp
#x                                                            CHOICES endx_nopsp_nousp
#x                                                            CHOICES endx_psp
#x                                                            CHOICES endx_usp
#x                                                            CHOICES endx_psp_usp
#x                                                            CHOICES enddx6
#x                                                            CHOICES enddt6
#x                                                            CHOICES endotp
#x                                                            CHOICES custom]
#x       [-srv6_adj_include_srv6_sid_structure_sub_sub_tlv    CHOICES 0 1]
#x       [-srv6_adj_locator_block_length                      RANGE 0-128
#x                                                            DEFAULT 40]
#x       [-srv6_adj_locator_node_length                       RANGE 0-128
#x                                                            DEFAULT 24]
#x       [-srv6_adj_function_length                           RANGE 0-128
#x                                                            DEFAULT 16]
#x       [-srv6_adj_argument_length                           RANGE 0-128
#x                                                            DEFAULT 0]
#x       [-srv6_adj_advertise_custom_sub_tlv                  CHOICES 0 1]
#x       [-srv6_adj_custom_sub_tlv                            HEX]
#x       [-interface_custom_end_point_function                NUMERIC]
#x       [-active_to                                          CHOICES 0 1]
#x       [-ipv6_adj_sid_to                                    IPV6]
#x       [-b_flag_to                                          CHOICES 0 1]
#x       [-s_flag_to                                          CHOICES 0 1]
#x       [-p_flag_to                                          CHOICES 0 1]
#x       [-reserved_to                                        HEX]
#x       [-interface_algorithm_to                             RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-weight_to                                          RANGE 0-255
#x                                                            DEFAULT 0]
#x       [-pseudo_srv6_end_point_function_to                  CHOICES end_nopsp_nousp
#x                                                            CHOICES end_psp
#x                                                            CHOICES end_usp
#x                                                            CHOICES end_psp_usp
#x                                                            CHOICES endt_nopsp_nousp
#x                                                            CHOICES endt_psp
#x                                                            CHOICES endt_usp
#x                                                            CHOICES endt_psp_usp
#x                                                            CHOICES endx_nopsp_nousp
#x                                                            CHOICES endx_psp
#x                                                            CHOICES endx_usp
#x                                                            CHOICES endx_psp_usp
#x                                                            CHOICES enddx6
#x                                                            CHOICES enddt6
#x                                                            CHOICES endotp
#x                                                            CHOICES custom]
#x       [-custom_end_point_function_to                       NUMERIC]
#x       [-srv6_adj_include_srv6_sid_structure_sub_sub_tlv_to CHOICES 0 1]
#x       [-srv6_adj_locator_block_length_to                   RANGE 0-128
#x                                                            DEFAULT 40]
#x       [-srv6_adj_locator_node_length_to                    RANGE 0-128
#x                                                            DEFAULT 24]
#x       [-srv6_adj_function_length_to                        RANGE 0-128
#x                                                            DEFAULT 16]
#x       [-srv6_adj_argument_length_to                        RANGE 0-128
#x                                                            DEFAULT 0]
#x       [-srv6_adj_advertise_custom_sub_tlv_to               CHOICES 0 1]
#x       [-srv6_adj_custom_sub_tlv_to                         HEX]
#x       [-custom_end_point_function                          NUMERIC]
#
# Arguments:
#    -mode
#        This option defines the action to be taken on the ISIS Router.
#    -handle
#        ISIS session handle for using the modes create and modify.
#        When -handle is provided with a protocol stack handle or a protocol session handle, the arguments
#        that configure global settings will only accept simple values. In this situation, these arguments will
#        configure only the settings of the parent device group or the ports associated with the parent topology.
#x   -locator_count
#x       Number of ISIS SRv6 Locator Entry per router.
#x   -interface_adj_si_count
#x       Number of ISIS Adjacent SIDs per interface.
#x   -locator_entry_active
#x       Activate/Deactivate Configuration.
#x   -srv6_metric
#x       Metric Value ranging from 0 to 4294967295.
#x   -srv6_algorithm
#x       Algorithm, integer value, ranging from 0 to 255.
#x   -d_bit
#x       When the Locator is leaked from level-2 to level-1, the D
#x       bit MUST be set.Otherwise, this bit MUST be clear.Locators
#x       with the D bit set MUST NOT be leaked from level-1 to level-2.
#x   -reserved_flags
#x       Reserved (Flags)
#x   -locator_size
#x       1 octet. Number of bits in the Locator field.
#x   -srv6_locator
#x       IPv6 address of Locator.
#x   -advertise_locator_as_prefix
#x       If enabled, it will advertise Locator as Prefix
#x   -prefix_length
#x       Prefix Length.
#x   -route_metric
#x       Route Metric, integer value, ranging from 0 to 16777215.
#x   -srv6_redistribution
#x       Redistribution type, valid values Up and Down.
#x   -route_origin
#x       Route Origin Type, valid values are Internal or External.
#x   -sid_count
#x       Number of SID in Locator Entry
#x   -srv6_end_sid_active
#x       Activate/Deactivate Configuration.
#x   -flags
#x       Flags, in 1 octet.
#x   -end_point_function
#x       End-Point Function, integer value ranging from 0 to 255.
#x   -pseudo_srv6_end_point_function
#x       a set of well-known functions that can be
#x       associated with a SID.
#x   -sid
#x       Segment Identifier.
#x   -include_srv6_sid_structure_sub_sub_tlv
#x       If enabled, it will include SRv6 SID Structure Sub-Sub-TLV
#x   -locator_block_length
#x       Locator Block Length in Bits.
#x   -locator_node_length
#x       Locator Node Length in Bits.
#x   -function_length
#x       Function Length in Bits.
#x   -argument_length
#x       Argument Length in Bits.
#x   -advertise_custom_sub_tlv
#x       If enabled, it will advertise Custom Sub-TLV.
#x   -custom_sub_tlv
#x       Define a Custom Sub-TLV.
#x   -srv6_adj_sid_active
#x       Activate/Deactivate Configuration.
#x   -ipv6_adj_sid
#x       IPv6 Adjacent Segment Identifier.
#x   -b_flag
#x       Backup flag, If set, the End.X SID is eligible for protection.
#x   -s_flag
#x       Set flag.When set, the S-Flag indicates that the End.X SID refers to a set of adjacencies (and therefore MAY be assigned to other adjacencies as well).
#x   -p_flag
#x       Persistent flag.When set, the P-Flag indicates that the End.X SID is persistently allocated, i.e., the End.X SID value remains consistent across router restart and/or interface flap.
#x   -reserved
#x       Reserved, MUST be zero when originated and ignored when received.
#x   -interface_algorithm
#x       1 octet. Associated algorithm.
#x   -weight
#x       1 octet.The value represents the weight of the End.X SID for the purpose of load balancing.
#x   -srv6_adj_end_point_function
#x       SRv6 Adjacent End-Point Function.
#x   -pseudo_srv6_adj_end_point_function
#x       SRv6 adjacent End-Point Function Type.
#x   -srv6_adj_include_srv6_sid_structure_sub_sub_tlv
#x       If enabled, it include SRv6 SID Structure Sub-Sub-TLV
#x   -srv6_adj_locator_block_length
#x       Locator Block Length in Bits.
#x   -srv6_adj_locator_node_length
#x       Locator Node Length in Bits.
#x   -srv6_adj_function_length
#x       Function Length in Bits.
#x   -srv6_adj_argument_length
#x       Argument Length in Bits.
#x   -srv6_adj_advertise_custom_sub_tlv
#x       If enabled, it will advertise Custom Sub-TLV.
#x   -srv6_adj_custom_sub_tlv
#x       Define Custom Sub-TLV.
#x   -interface_custom_end_point_function
#x       Define Custom End-Point Function.
#x   -active_to
#x       Activate/Deactivate Configuration.
#x   -ipv6_adj_sid_to
#x       IPv6 Adjacent Segment Identifier.
#x   -b_flag_to
#x       Backup flag, If set, the End.X SID is eligible for protection.
#x   -s_flag_to
#x       Set flag.When set, the S-Flag indicates that the End.X SID refers to a set of adjacencies (and therefore MAY be assigned to other adjacencies as well).
#x   -p_flag_to
#x       Persistent flag.When set, the P-Flag indicates that the End.X SID is persistently allocated, i.e., the End.X SID value remains consistent across router restart and/or interface flap.
#x   -reserved_to
#x       Reserved, MUST be zero when originated and ignored when received.
#x   -interface_algorithm_to
#x       1 octet. Associated algorithm.
#x   -weight_to
#x       1 octet.The value represents the weight of the End.X SID for the purpose of load balancing.
#x   -pseudo_srv6_end_point_function_to
#x       SRv6 End-Point Function Type.
#x   -custom_end_point_function_to
#x       Define Custom End-Point Function.
#x   -srv6_adj_include_srv6_sid_structure_sub_sub_tlv_to
#x       If enabled, it include SRv6 SID Structure Sub-Sub-TLV.
#x   -srv6_adj_locator_block_length_to
#x       Locator Block Length in Bits.
#x   -srv6_adj_locator_node_length_to
#x       Locator Node Length in Bits.
#x   -srv6_adj_function_length_to
#x       Function Length in Bits.
#x   -srv6_adj_argument_length_to
#x       Argument Length in Bits.
#x   -srv6_adj_advertise_custom_sub_tlv_to
#x       if enabled, it will advertise Custom Sub-TLV.
#x   -srv6_adj_custom_sub_tlv_to
#x       Define Custom Sub-TLV.
#x   -custom_end_point_function
#x       Define Custom End-Point Function.
#
# Return Values:
#    A list containing the isis interface srv6 protocol stack handles that were added by the command (if any).
#x   key:isis_interface_srv6_handle         value:A list containing the isis interface srv6 protocol stack handles that were added by the command (if any).
#    A list containing the isis router srv6 protocol stack handles that were added by the command (if any).
#x   key:isis_router_srv6_handle            value:A list containing the isis router srv6 protocol stack handles that were added by the command (if any).
#    A list containing the isis pseudo interface srv6 protocol stack handles that were added by the command (if any).
#x   key:isis_pseudo_interface_srv6_handle  value:A list containing the isis pseudo interface srv6 protocol stack handles that were added by the command (if any).
#    A list containing the isis pseudo router srv6 protocol stack handles that were added by the command (if any).
#x   key:isis_pseudo_router_srv6_handle     value:A list containing the isis pseudo router srv6 protocol stack handles that were added by the command (if any).
#    $::SUCCESS | $::FAILURE
#    key:status                             value:$::SUCCESS | $::FAILURE
#    On status of failure, gives detailed information.
#    key:log                                value:On status of failure, gives detailed information.
#    list of SRv6 node handles
#    key:handle                             value:list of SRv6 node handles
#
# Examples:
#
# Sample Input:
#
# Sample Output:
#
# Notes:
#
# See Also:
#

package ixiangpf;

use utils;
use ixiahlt;

sub emulation_isis_srv6_config {

	my $args = shift(@_);

	my @notImplementedParams = ();
	my @mandatoryParams = ();
	my @fileParams = ();

	# ixiahlt::logHltapiCommand('emulation_isis_srv6_config', $args);
	# ixiahlt::utrackerLog ('emulation_isis_srv6_config', $args);

	return ixiangpf::runExecuteCommand('emulation_isis_srv6_config', \@notImplementedParams, \@mandatoryParams, \@fileParams, $args);
}

# Return value for the package
return 1;
