# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def emulation_mka_info(self, mode, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    emulation_mka_info
		
		 Description:
		    This procedure retrieves information about the MKA sessions.
		    This procedure is also used to fetch stats, learned information and configured properties of MKA, depending on the given mode and handle.
		
		 Synopsis:
		    emulation_mka_info
		        [-handle ANY]
		        -mode    CHOICES per_port_stats
		                 CHOICES per_device_group_stats
		                 CHOICES per_session_stats
		                 CHOICES clear_stats
		                 CHOICES learned_info
		                 CHOICES clear_learned_info
		
		 Arguments:
		    -handle
		        The routers from which to extract MKA data.
		        One of the two parameters is required: port_handle/handle.
		    -mode
		        The action that should be taken.
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		    key:status                                                    value:$::SUCCESS | $::FAILURE
		    If status is failure, detailed information provided.
		    key:log                                                       value:If status is failure, detailed information provided.
		    Sessions Up
		x   key:sessions_up                                               value:Sessions Up
		    Sessions Down
		x   key:sessions_down                                             value:Sessions Down
		    Sessions Not Started
		x   key:sessions_not_started                                      value:Sessions Not Started
		    Sessions Total
		x   key:sessions_total                                            value:Sessions Total
		    MKPDU Tx
		x   key:mkpdu_tx                                                  value:MKPDU Tx
		    MKPDU Rx
		x   key:mkpdu_rx                                                  value:MKPDU Rx
		    Live Peer Count
		x   key:live_peer_count                                           value:Live Peer Count
		    Potential Peer Count
		x   key:potential_peer_count                                      value:Potential Peer Count
		    Latest Key Tx Peer Count
		x   key:latest_key_tx_peer_count                                  value:Latest Key Tx Peer Count
		    Latest Key Rx Peer Count
		x   key:latest_key_rx_peer_count                                  value:Latest Key Rx Peer Count
		    Malformed Mkpdu
		x   key:malformed_mkpdu                                           value:Malformed Mkpdu
		    ICV Mismatch
		x   key:icv_mismatch                                              value:ICV Mismatch
		    ICV Key.
		    key:<handle>.<learned_info>.icv_key                           value:ICV Key.
		    Key Encrypting Key.
		    key:<handle>.<learned_info>.key_encrypting_key                value:Key Encrypting Key.
		    Secure Association Key.
		    key:<handle>.<learned_info>.secure_association_key            value:Secure Association Key.
		    12 bytes Salt value for XPN cipher suites.
		    key:<handle>.<learned_info>.salt                              value:12 bytes Salt value for XPN cipher suites.
		    Live Peer Member Identifier.
		    key:<handle>.<learned_info>.live_peer_member_identifier       value:Live Peer Member Identifier.
		    Live Peer Message Number.
		    key:<handle>.<learned_info>.live_peer_message_number          value:Live Peer Message Number.
		    Potential Peer Member Identifier.
		    key:<handle>.<learned_info>.potential_peer_member_identifier  value:Potential Peer Member Identifier.
		    Potential Peer Message Number.
		    key:<handle>.<learned_info>.potential_peer_message_number     value:Potential Peer Message Number.
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'emulation_mka_info', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
