################################################################################
#                                                                              #
#    Copyright 1997 - 2022 by IXIA  Keysight                                   #
#    All Rights Reserved.                                                      #
#                                                                              #
################################################################################

################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

################################################################################
#                                                                              #
# Description:                                                                 #
#    Script  will create following:                                            #
#    1. Create TWAMPv4 devices                                                 #
#    2. Set the host mode to Sender for first device group                     #
#    3.	Set the TWAMPv4 Destination address                                    #
#    4.	Assign the ports                                                       #
#    5. Start all protocols                           						   # 
#    6. Retrieve protocol statistics                                           #
#    7. Execute send request from sender device                                #
#  	 8. Check stats        										               #
#    9. Stop all protocols	   					  							   #
#                                                                              #
################################################################################

if 'py' not in dir():
    class TestFailedError(Exception): pass
    class Py: pass
    py = Py()
    py.ports = [('10.39.64.116', 2, 5), ('10.39.64.116', 2, 6)]
    py.ixTclServer = '10.39.64.152'
    py.ixTclPort = 7771

################################################################################
# Import the IxNet library
################################################################################
import IxNetwork
import time
ixNet = IxNetwork.IxNet()

################################################################################
# Connect to IxNet client
################################################################################

ixNet.connect(py.ixTclServer, '-port', py.ixTclPort, '-version', '9.30')

################################################################################
# Cleaning up IxNetwork
################################################################################
print ("Cleaning up IxNetwork...")
ixNet.execute('newConfig')

################################################################################
# Adding ports to configuration
################################################################################
print ("Adding ports to configuration")
root = ixNet.getRoot()
ixNet.add(root, 'vport')
ixNet.add(root, 'vport')
ixNet.commit()
vPorts = ixNet.getList(root, 'vport')
vport1 = vPorts[0]
vport2 = vPorts[1]

################################################################################
# Adding TWAMPv4 Protocol
################################################################################

print ("Add topologies")
ixNet.add(root, 'topology')
ixNet.add(root, 'topology')
ixNet.commit()

topo1 = ixNet.getList(root, 'topology')[0]
topo2 = ixNet.getList(root, 'topology')[1]

print ("Add ports to topologies")
ixNet.setAttribute(topo1, '-vports', vport1)
ixNet.setAttribute(topo2, '-vports', vport2)
ixNet.commit()

print ("Add device groups to topologies")
ixNet.add(topo1, 'deviceGroup')
ixNet.add(topo2, 'deviceGroup')
ixNet.commit()

dg1 = ixNet.getList(topo1, 'deviceGroup')[0]
dg2 = ixNet.getList(topo2, 'deviceGroup')[0]

print ("Add Ethernet stacks to device groups")
ixNet.add(dg1, 'ethernet')
ixNet.add(dg2, 'ethernet')
ixNet.commit()

mac1 = ixNet.getList(dg1, 'ethernet')[0]
mac2 = ixNet.getList(dg2, 'ethernet')[0]

print ("Add ipv4 stacks to Ethernets")
ixNet.add(mac1, 'ipv4')
ixNet.add(mac2, 'ipv4')
ixNet.commit()

ipv4_1 = ixNet.getList(mac1, 'ipv4')[0]
ipv4_2 = ixNet.getList(mac2, 'ipv4')[0]

print ("Setting multi values for ipv4 addresses")
ixNet.setMultiAttribute(ixNet.getAttribute(ipv4_1, '-address') + '/counter', '-start', '22.1.1.1', '-step', '0.0.1.0')
ixNet.setMultiAttribute(ixNet.getAttribute(ipv4_1, '-gatewayIp') + '/counter', '-start', '22.1.1.2', '-step', '0.0.1.0')
ixNet.setMultiAttribute(ixNet.getAttribute(ipv4_1, '-resolveGateway') + '/singleValue', '-value', 'true')
ixNet.setMultiAttribute(ixNet.getAttribute(ipv4_2, '-address') + '/counter', '-start', '22.1.1.2', '-step', '0.0.1.0')
ixNet.setMultiAttribute(ixNet.getAttribute(ipv4_2, '-gatewayIp') + '/counter', '-start', '22.1.1.1', '-step', '0.0.1.0')
ixNet.setMultiAttribute(ixNet.getAttribute(ipv4_2, '-resolveGateway') + '/singleValue', '-value', 'true')
ixNet.commit()

print ("Add TWAMP stacks to IPv4")
ixNet.add(ipv4_1, 'twampIpv4')
ixNet.add(ipv4_2, 'twampIpv4')
ixNet.commit()

### Similarly TWAMPv6 devices can be created, by adding IPv6 stacks and adding TWAMPv6 stacks on top of that ####

twamp_sender = ixNet.getList(ipv4_1, 'twampIpv4')[0]
twamp_reflector = ixNet.getList(ipv4_2, 'twampIpv4')[0]

############# Set the TWAMP Hostmode to sender for first device group  ##############
print ("Set the TWAMP Hostmode to sender for first device group")
host_mode = ixNet.getAttribute(twamp_sender, '-hostMode')
ixNet.setAttribute(host_mode, '-pattern', 'singleValue')
ixNet.commit()
host_mode_single_val = ixNet.getList(host_mode, 'singleValue')[0]
ixNet.setAttribute(host_mode_single_val, '-value', 'sender')
ixNet.commit()

############# Set Number of Destinations to 1 on Sender ######
print ("Set Number of Destinations to 1 on Sender")
ixNet.setAttribute(twamp_sender, '-noOfDest', 1)
ixNet.commit()

############# Set  TWAMP IPv4 Destination Addresses on Sender ######
print ("Set  TWAMP IPv4 Destination Addresses on Sender ")
twamp_ipv4_dst_list = ixNet.getList(twamp_sender, 'twampIpv4DstList')[0]
ixNet.setMultiAttribute(ixNet.getAttribute(twamp_ipv4_dst_list, '-ipv4DestAddress') + '/counter', '-start', '22.1.1.2', '-step', '0.0.1.0')
ixNet.commit()

# ################################################################################
# # Assign ports
# ################################################################################
vports = ixNet.getList(ixNet.getRoot(), 'vport')
print ("Assigning ports to " + str(vports) + " ...")
assignPorts = ixNet.execute('assignPorts', py.ports, [], ixNet.getList("/","vport"), True)
if assignPorts != vports:
    raise TestFailedError("FAILED assigning ports. Got %s" %assignPorts)
else:
    print("PASSED assigning ports. Got %s" %assignPorts)

# ################################################################################
# # Start All Protocols
# ################################################################################
print ("Starting All Protocols")
ixNet.execute('startAllProtocols')
print ("Sleep 30sec for protocols to start")
time.sleep(30)

################################################################################
# Retrieve protocol statistics.
################################################################################
print ("Fetching all Protocol Summary Stats\n")
viewPage = '::ixNet::OBJ-/statistics/view:"Protocols Summary"/page'
statcap = ixNet.getAttribute(viewPage, '-columnCaptions')
for statValList in ixNet.getAttribute(viewPage, '-rowValues'):
    for statVal in statValList:
        print("***************************************************")
        index = 0
        for satIndv in statVal:
            print("%-30s:%s" % (statcap[index], satIndv))
            index = index + 1
        # end for
    # end for
# end for
print("***************************************************")

############# Execute Send request from Sender device ######
print ("Execute Send request from Sender device")
ixNet.execute('sendTwampRequest',twamp_ipv4_dst_list,(1,2,3,4,5,6,7,8,9,10))
time.sleep(5)

print ("Fetching TWAMPv4 Protocol per port stats \n")
viewPage = '::ixNet::OBJ-/statistics/view:"TWAMPv4 Per Port"/page'
statcap = ixNet.getAttribute(viewPage, '-columnCaptions')
for statValList in ixNet.getAttribute(viewPage, '-rowValues'):
    for statVal in statValList:
        print("***************************************************")
        index = 0
        for satIndv in statVal:
            print("%-30s:%s" % (statcap[index], satIndv))
            index = index + 1
print("***************************************************")

################################################################################
#  Stop all protocols                                                          #
################################################################################
print ('Stopping protocols')
ixNet.execute('stopAllProtocols')
time.sleep(10)

print ("\n\nCleaning up IxNetwork...")
ixNet.execute('newConfig')
print ('!!! Test Script Ends !!!')